﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Trolleybus.DrawingObjects;
using Trolleybus.MovementStrategy;

namespace Trolleybus.Generics
{
    internal class BusesGenericStorage
    {
        /// <summary>
        /// Словарь (хранилище)
        /// </summary>
        readonly Dictionary<string, BusesGenericCollection<DrawingBus, DrawingObjectBus>> _busStorages;

        /// <summary>
        /// Возвращение списка названий наборов
        /// </summary>
        public List<string> Keys => _busStorages.Keys.ToList();

        /// <summary>
        /// Ширина окна отрисовки
        /// </summary>
        private readonly int _pictureWidth;

        /// <summary>
        /// Высота окна отрисовки
        /// </summary>
        private readonly int _pictureHeight;

        /// <summary>
        /// Разделитель для записи ключа и значения элемента словаря
        /// </summary>
        private static readonly char _separatorForKeyValue = '|';
        /// <summary>
        /// Разделитель для записей коллекции данных в файл
        /// </summary>
        private readonly char _separatorRecords = ';';
        /// <summary>
        /// Разделитель для записи информации по объекту в файл
        /// </summary>
        private static readonly char _separatorForObject = ':';

        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="pictureWidth"></param>
        /// <param name="pictureHeight"></param>
        public BusesGenericStorage(int pictureWidth, int pictureHeight)
        {
            _busStorages = new Dictionary<string, BusesGenericCollection<DrawingBus, DrawingObjectBus>>();
            _pictureWidth = pictureWidth;
            _pictureHeight = pictureHeight;
        }

        /// <summary>
        /// Добавление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void AddSet(string name)
        {
            // проверка, существует ли набор с таким ключём
            if (_busStorages.ContainsKey(name))
            {
                return;
            }
            var busCollection = new BusesGenericCollection<DrawingBus, DrawingObjectBus>(_pictureWidth, _pictureHeight);
            _busStorages.Add(name, busCollection);
        }

        /// <summary>
        /// Удаление набора
        /// </summary>
        /// <param name="name">Название набора</param>
        public void DelSet(string name)
        {
            if (_busStorages.ContainsKey(name))
            {
                _busStorages.Remove(name);
            }
        }
        /// <summary>
        /// Доступ к набору
        /// </summary>
        /// <param name="ind"></param>
        /// <returns></returns>
        public BusesGenericCollection<DrawingBus, DrawingObjectBus>? this[string ind]
        {
            get
            {
                if (!_busStorages.ContainsKey(ind))
                {
                    return null;
                }
                return _busStorages[ind];
            }
        }
        /// <summary>
        /// Сохранение информации по автобусам из хранилища в файл
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - сохранение прошло успешно, false - ошибка при сохранении данных</returns>
        public bool SaveData(string filename)
        {
            if (File.Exists(filename))
            {
                File.Delete(filename);
            }

            StringBuilder data = new();
            foreach (KeyValuePair<string, BusesGenericCollection<DrawingBus, DrawingObjectBus>> record in _busStorages)
            {
                StringBuilder records = new();
                foreach (DrawingBus? elem in record.Value.GetBuses)
                {
                    records.Append($"{elem?.GetDataForSave(_separatorForObject)}{_separatorRecords}");
                }
                data.AppendLine($"{record.Key}{_separatorForKeyValue}{records}");
            }
            if (data.Length == 0)
            {
                return false;
            }
            using StreamWriter sw = new StreamWriter(filename, false);
            sw.WriteLine($"BusesStorage");
            sw.WriteLine(data);
            return true;
        }
        /// <summary>
        /// Загрузка информации по автобусам из файла в хранилище
        /// </summary>
        /// <param name="filename">Путь и имя файла</param>
        /// <returns>true - загрузка прошла успешно, false - ошибка при загрузке данных</returns>
        public bool LoadData(string filename)
        {
            if (!File.Exists(filename))
            {
                return false;
            }
            using(StreamReader sr = File.OpenText(filename))
            {
                String currentLine = sr.ReadLine();
                // Если первая строка пустая или в файле не те данные
                if (currentLine == null || !currentLine.Contains("BusesStorage"))
                {
                    return false;
                }
                // Очистка хранилища
                _busStorages.Clear();
                currentLine = sr.ReadLine();
                while (currentLine != "" && currentLine != null)
                {
                    string[] record = currentLine.Split(_separatorForKeyValue, StringSplitOptions.RemoveEmptyEntries);
                    BusesGenericCollection<DrawingBus, DrawingObjectBus> collection = new(_pictureWidth, _pictureHeight);
                    string[] set = record[1].Split(_separatorRecords, StringSplitOptions.RemoveEmptyEntries);
                    foreach (string elem in set)
                    {
                        DrawingBus? bus = elem?.CreateDrawingBus(_separatorForObject, _pictureWidth, _pictureHeight);
                        if (bus != null)
                        {
                            if ((collection + bus) == -1)
                            {
                                return false;
                            }
                        }
                    }
                    _busStorages.Add(record[0], collection);
                    currentLine = sr.ReadLine();
                }
            }
            return true;          
        }
    }
}
